<?php

// {"Params":{"Users":["1"],"IncludeDeactivatedForms":false,"SelectFields":["ShortName","Description"]}}
class CheckStudip extends StudIPPlugin implements SystemPlugin
{
    static $metrics = null;
    static $max_slots = 20;
    static $minutes = 10;

    public function __construct()
    {
        parent::__construct();
        StudipAutoloader::addAutoloadPath(__DIR__ . '/lib');
        if ($GLOBALS['perm']->have_perm("root")) {
            if (CheckRemoteServer::exists(2)) {
                $statement = DBManager::get()->prepare("
                    SELECT pluginid FROM plugins WHERE pluginclassname = 'CheckStudip'
                ");
                $statement->execute();
                $plugin_id = $statement->fetch(PDO::FETCH_COLUMN, 0);

                $statement = DBManager::get()->prepare("
                    INSERT IGNORE INTO roles_plugins
                    SET roleid = 7,
                        pluginid = :plugin_id
                ");
                $statement->execute([
                    'plugin_id' => $plugin_id
                ]);
                CheckRemoteServer::deleteBySQL('server_id = 2');
            }
        }
        if ($GLOBALS['perm']->have_perm("root") && !UpdateInformation::isCollecting()) {
            $nav = new Navigation(
                _("Stud.IP überprüfen"),
                PluginEngine::getURL($this, array(), "check/all")
            );
            Navigation::addItem("/admin/locations/checkstudip", $nav);

            $space = disk_free_space($GLOBALS['TMP_PATH']);
            $min = Config::get()->DISK_FREE_SPACE_ALERT;
            if ($min && $space < $min) {
                $val = self::getDiscSpaceString($space);
                PageLayout::postInfo(sprintf(_("Nur noch %s an Speicher im Temp-Ordner frei."), $val));
            }
        }
        if (self::displayMonitor()) {
            $nav = new Navigation(
                _('Monitoring'),
                PluginEngine::getURL($this, [], 'monitor/dashboard')
            );
            $nav->setImage(Icon::create('outer-space', Icon::ROLE_NAVIGATION));
            Navigation::addItem('/monitor', $nav);
            $nav = new Navigation(_('Dashboard'), PluginEngine::getURL($this, [], 'monitor/dashboard'));
            Navigation::addItem('/monitor/dashboard', $nav);
            $nav = new Navigation(_('Soap-Abfragen'), PluginEngine::getURL($this, [], 'testsoap/overview'));
            Navigation::addItem('/monitor/soap', $nav);
            $nav = new Navigation(_('Administration'), PluginEngine::getURL($this, [], 'server/admin'));
            Navigation::addItem('/monitor/server', $nav);

            if (UpdateInformation::isCollecting()) {
                //$data = Request::getArray("page_info");
                if (stripos(Request::get("page"), "plugins.php/checkstudip") !== false) {
                    $servers = CheckRemoteServer::findBySQL('1=1 ORDER BY name ASC');
                    $active_alarm_ids = [];
                    foreach ($servers as $server) {
                        $lastresponse = CheckMonitoringData::findOneBySQL('`server_id` = ? ORDER BY `id` DESC LIMIT 1', [$server->id]);
                        if (isset($lastresponse) && $server['active']) {
                            if (!$lastresponse['success']) {
                                $active_alarm_ids[] = 'server_'.$server->id;
                            } else {
                                foreach ($server->alarms as $alarm) {
                                    if ($alarm['type'] !== 'error') {
                                        continue;
                                    }
                                    if (isset($lastresponse[$alarm['metric']])) {
                                        if ($alarm['compare'] === '>=') {
                                            if (floatval($lastresponse[$alarm['metric']]) >= floatval($alarm['value'])) {
                                                $active_alarm_ids[] = $alarm->id;
                                            }
                                        }

                                        if ($alarm['compare'] === '<=') {

                                            if (floatval($lastresponse[$alarm['metric']]) <= floatval($alarm['value'])) {
                                                $active_alarm_ids[] = $alarm->id;
                                            }
                                        }
                                        if ($alarm['compare'] === '!=') {
                                            if (trim($lastresponse[$alarm['metric']]) != trim($alarm['value'])) {
                                                $active_alarm_ids[] = $alarm->id;
                                            }
                                        }
                                        if ($alarm['compare'] === '=') {
                                            if (trim($lastresponse[$alarm['metric']]) == trim($alarm['value'])) {
                                                $active_alarm_ids[] = $alarm->id;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                    $tf = new Flexi_TemplateFactory(__DIR__.'/views');
                    $template = $tf->open('monitor/_table');
                    $template->minutes = 10;
                    $template->max_slots = CheckStudip::$max_slots;
                    $template->servers = CheckRemoteServer::findBySQL('1=1 ORDER BY name ASC');
                    $template->plugin = $this;
                    $get_server_data = DBManager::get()->prepare("
                        SELECT *
                        FROM check_monitoring_data
                        WHERE server_id = :server_id
                            AND mkdate >= :min_time
                            AND mkdate <= :max_time
                        LIMIT 1
                    ");
                    foreach ($template->servers as $server) {
                        $d = [];
                        for ($i = 0; $i < CheckStudip::$max_slots; $i++) {
                            $get_server_data->execute([
                                'server_id' => $server->getId(),
                                'min_time' => time() - ($i * CheckStudip::$minutes * 60) - (CheckStudip::$minutes * 60 / 2),
                                'max_time' => time() - ($i * CheckStudip::$minutes * 60) + (CheckStudip::$minutes * 60 / 2)
                            ]);
                            $row = $get_server_data->fetch(PDO::FETCH_ASSOC);
                            if ($row) {
                                $d[] = $row;
                            }
                        }
                        $data[$server->getId()] = $d;
                    }
                    $template->data = $data;
                    UpdateInformation::setInformation("CheckStudipDashboard", [
                        'active_alarm_ids' => $active_alarm_ids,
                        'data' => array_values(CheckStudip::getGraphData()),
                        'table' => $template->render()
                    ]);
                }
            }
        }
    }

    /**
     * @return CheckStudipMetric[]
     */
    static public function getMetrics()
    {
        if (self::$metrics === null) {
            self::$metrics = [];
            foreach (scandir(__DIR__.'/lib/metrics') as $file) {
                if ($file[0] !== '.') {
                    require_once __DIR__.'/lib/metrics/'.$file;
                }
            }
            foreach (get_declared_classes() as $class) {
                if (in_array('CheckStudipMetric', class_implements($class))) {
                    self::$metrics[$class] = new $class();
                }
            }
            uasort(self::$metrics, function ($a, $b) {
                return strcasecmp($a->getName(), $b->getName());
            });
        }
        return self::$metrics;
    }

    static public function displayMonitor()
    {
        if (Config::get()->CHECKSTUDIP_MONITOR_ROLE) {
            $display_monitor = false;
            if (in_array(Config::get()->CHECKSTUDIP_MONITOR_ROLE, ['nobody', 'user', 'autor', 'dozent', 'admin', 'root'])) {
                $display_monitor = $GLOBALS['perm']->have_perm(Config::get()->CHECKSTUDIP_MONITOR_ROLE);
            } elseif ($GLOBALS['perm']->have_perm('autor') && in_array(Config::get()->CHECKSTUDIP_MONITOR_ROLE, RolePersistence::getAssignedRoles(User::findCurrent()->id))) {
                $display_monitor = true;
            }
            return $display_monitor;
        }
        return false;
    }

    static public function collectData()
    {
        $output = [];
        $output['installation_name'] = Config::get()->UNI_NAME_CLEAN;

        foreach (self::getMetrics() as $metric) {
            $output[$metric->getIndex()] = $metric->measure();
        }

        return $output;
    }

    static public function getNumberOfCPUs() {
        $ans = 1;
        if (is_file('/proc/cpuinfo')) {
            $cpuinfo = file_get_contents('/proc/cpuinfo');
            preg_match_all('/^processor/m', $cpuinfo, $matches);
            $ans = count($matches[0]);
        } else if ('WIN' === strtoupper(substr(PHP_OS, 0, 3))) {
            $process = @popen('wmic cpu get NumberOfCores', 'rb');
            if (false !== $process) {
                fgets($process);
                $ans = intval(fgets($process));
                pclose($process);
            }
        } else {
            $ps = @popen('sysctl -a', 'rb');
            if ($ps !== false) {
                $output = stream_get_contents($ps);
                preg_match('/hw.ncpu: (\d+)/', $output, $matches);
                if ($matches) {
                    $ans = intval($matches[1]);
                }
                pclose($ps);
            }
        }
        return $ans;
    }


    static public function getDiscSpaceString($space)
    {
        if ($space > 1024 * 1024 * 1024 * 1024) {
            $number = round($space / (1024 * 1024 * 1024 * 1024), 2);
            $unit = "TB";
        } elseif ($space > 1024 * 1024) {
            $number = round($space / (1024 * 1024 * 1024), 2);
            $unit = "GB";
        } elseif ($space > 1024 * 1024) {
            $number = round($space / (1024 * 1024), 2);
            $unit = "MB";
        } else {
            $number = round($space / (1024), 2);
            $unit = "KB";
        }
        return $number.' '.$unit;
    }

    static public function getDefaultAlarms()
    {
        $alarms = [
            [
                'metric' => 'load_percentage',
                'compare' => '>=',
                'value' => 80,
                'type' => 'warning'
            ],
            [
                'metric' => 'nonwritable_folders',
                'compare' => '!=',
                'value' => '',
                'type' => 'error'
            ]
        ];
        foreach (CheckStudip::getMetrics() as $metric) {
            $alarm_data = $metric->defaultAlarm();
            if ($alarm_data) {
                $alarms[] = [
                    'metric' => $metric->getIndex(),
                    'compare' => $alarm_data['compare'],
                    'value' => $alarm_data['value'],
                    'type' => $alarm_data['type']
                ];
            }
        }
        return $alarms;
    }

    static public function getGraphData() {
        $index = UserConfig::get($GLOBALS['user']->id)->CHECK_SELECTED_METRIC ?? 'users_online'; //load_percentage, mycourses_time
        $servers = CheckRemoteServer::findBySQL('1=1 ORDER BY name ASC');
        $series = [];
        $server_ids = [];
        foreach ($servers as $server) {
            $server_ids[] = $server->id;
            $series[$server->id] = [
                'name' => $server['name'],
                'data' => []
            ];
        }
        $points = CheckMonitoringPoint::findBySQL('`planned_time` >= UNIX_TIMESTAMP() - :slots ORDER BY `planned_time` ASC', [
            'slots' => CheckStudip::$max_slots * 60 * 10
        ]);
        foreach ($points as $point) {
            $point_server_ids = [];
            foreach ($point->data as $data) {
                $point_server_ids[] = $data->server_id;
                $series[$data->server_id]['data'][] = $data[$index];
            }
            foreach (array_diff($server_ids, $point_server_ids) as $server_id) {
                $series[$server_id]['data'][] = 0;
            }
        }

        return $series;

    }
}
