<?php

class CheckController extends PluginController
{
    public function all_action()
    {
        Navigation::activateItem("/admin/locations/checkstudip");
        if (!$GLOBALS['perm']->have_perm("root")) {
            throw new AccessDeniedException();
        }
        $this->results = array();
        $output = CheckStudip::collectData();

        $space = disk_free_space($GLOBALS['TMP_PATH']);
        $bytes_in_tmp = $this->getConsumedBytesInPath($GLOBALS['TMP_PATH']);
        $min = Config::get()->DISK_FREE_SPACE_ALERT;
        $this->tmp_free_space = $this->getHumanReadableSize($space);
        $this->bytes_in_tmp = $this->getHumanReadableSize($bytes_in_tmp);

        if ($min && $space < $min) {
            $this->results[] = array(
                'type' => "info",
                'message' => sprintf(_("Nur noch %s an Speicher im Temp-Ordner frei."), $this->tmp_free_space)
            );
        }

        $writable_folders = array(
            $GLOBALS['UPLOAD_PATH'],  // "data/user_doc"
            $GLOBALS['PLUGIN_ASSETS_PATH'], //"data/assets_cache"
            $GLOBALS['PLUGINS_PATH'], //"public/plugins_packages"
            $GLOBALS['DYNAMIC_CONTENT_PATH'], // "public/pictures"
            $GLOBALS['DYNAMIC_CONTENT_PATH']."/course",
            $GLOBALS['DYNAMIC_CONTENT_PATH']."/user",
            $GLOBALS['DYNAMIC_CONTENT_PATH']."/institute",
            $GLOBALS['DYNAMIC_CONTENT_PATH']."/loginbackgrounds"
        );
        foreach ($writable_folders as $folder) {
            $folderpath = $folder[0] === "/" ? $folder : $GLOBALS['STUDIP_BASE_PATH']."/".$folder;
            if (!is_writable($folderpath)) {
                $this->results[] = array(
                    'type' => "error",
                    'message' => sprintf(_("Ordner %s ist nicht schreibbar."), $folder)
                );
            }
        }

        foreach (CheckStudip::getMetrics() as $metric) {
            $alarm_data = $metric->defaultAlarm();
            if ($alarm_data) {
                $value = $output[$metric->getIndex()];
                switch ($alarm_data['compare']) {
                    case '!=':
                        if ($value != $alarm_data['value']) {
                            $this->results[] = array(
                                'type' => $alarm_data['type'],
                                'message' => $metric->getMessage()
                            );
                        }
                        break;
                    case '<=':
                        if ($value <= $alarm_data['value']) {
                            $this->results[] = array(
                                'type' => $alarm_data['type'],
                                'message' => $metric->getMessage()
                            );
                        }
                        break;
                    case '>=':
                        if ($value >= $alarm_data['value']) {
                            $this->results[] = array(
                                'type' => $alarm_data['type'],
                                'message' => $metric->getMessage()
                            );
                        }
                        break;
                    case '=':
                        if ($value == $alarm_data['value']) {
                            $this->results[] = array(
                                'type' => $alarm_data['type'],
                                'message' => $metric->getMessage()
                            );
                        }
                        break;
                }
            }
        }

        $this->report = $output;

    }

    public function error_log_action()
    {
        Navigation::activateItem("/admin/locations/checkstudip");
        if (!$GLOBALS['perm']->have_perm("root")) {
            throw new AccessDeniedException();
        }
        $months = [
            'Jan' => '01',
            'Feb' => '02',
            'Mar' => '03',
            'Apr' => '04',
            'May' => '05',
            'Jun' => '06',
            'Jul' => '07',
            'Aug' => '08',
            'Sep' => '09',
            'Oct' => '10',
            'Nov' => '11',
            'Dec' => '12'
        ];
        PageLayout::setTitle(_('Error-Log'));
        if (ini_get('error_log') || $GLOBALS['ERROR_LOG_PATH']) {
            $this->log = file_get_contents($GLOBALS['ERROR_LOG_PATH'] ?? ini_get('error_log'));
            $this->errors = [];
            $error_types = [];
            foreach (array_reverse(explode("\n", $this->log)) as $key => $error) {
                if ($error) {
                    preg_match("/\[([^\]]+)\]\s+\[([^\]]+)\]\s+\[([^\]]+)\]\s+(\[([^\]]+)\])?(.*)/", $error, $matches);
                    $message = $matches[6] ?? '';
                    $message = str_replace("\\n", "\n", $message);
                    $datetime = strtotime($matches[1] ?? '');
                    if ($datetime === false && isset($matches[1])) {
                        preg_match("/(\w{3})\s+(\w{3})\s+(\d\d)\s+(\d\d):(\d\d):(\d\d)\.(\d+)\s(\d{4})/", $matches[1], $matches2);
                        $datetime = strtotime($matches2[8].'-'.$months[$matches2[2]].'-'.$matches2[3].' '.$matches2[4].':'.$matches2[5].':'.$matches2[6]);
                    }
                    if (!is_int($datetime) || $datetime > time() - 86400 * 2) {
                        $error_type_id = md5(($matches[2] ?? '').$message);
                        if (isset($error_types[$error_type_id])) {
                            $error_types[$error_type_id]++;
                        } else {
                            $error_types[$error_type_id] = 1;
                        }
                        $this->errors[] = [
                            'datetime' => $datetime ?? $matches[1],
                            'level' => $matches[2] ?? '',
                            'process' => $matches[3] ?? '',
                            'client' => $matches[4] ?? '',
                            'message' => $message,
                            'original' => $error,
                            'cardinality' => 1
                        ];
                    }
                }
            }
            foreach ($this->errors as $i => $error) {
                $error_type_id = md5($error['level'].$error['message']);
                $this->errors[$i]['cardinality'] = $error_types[$error_type_id];
            }
        } else {
            PageLayout::postInfo(_('Konnte nicht auf das Error-Log des Servers zugreifen.'));
        }
    }

    public function phpinfo_action()
    {
        Navigation::activateItem("/admin/locations/checkstudip");
        if (!$GLOBALS['perm']->have_perm("root")) {
            throw new AccessDeniedException();
        }
        phpinfo();
        $this->render_nothing();
    }

    public function querylog_action()
    {
        Navigation::activateItem("/admin/locations/checkstudip");
        $statement = DBManager::get()->prepare("
            SELECT * FROM INFORMATION_SCHEMA.PROCESSLIST WHERE `DB` = :db ORDER BY `TIME` DESC
        ");
        $statement->execute([
            'db' => $GLOBALS['DB_STUDIP_DATABASE']
        ]);
        $this->results = $statement->fetchAll(PDO::FETCH_ASSOC);
    }

    public function killdbprocess_action()
    {
        if (!Request::isPost()) {
            throw new AccessDeniedException();
        }
        $failed = false;
        $statement = DBManager::get()->prepare("
            KILL :process_id
        ");
        try {
            $statement->execute([
                'process_id' => Request::get('pid')
            ]);
        } catch (Exception $e) {
            $failed = true;
            PageLayout::postInfo(_('Prozess konnte nicht gelöscht werden.'), [$e->getMessage()]);
        }
        if (!$failed) {
            PageLayout::postSuccess(_('Prozess wurde gelöscht.'));
        }
        $this->redirect('check/querylog');
    }

    protected function getConsumedBytesInPath($path)
    {
        $size = 0;
        foreach (scandir($path) as $file_or_folder) {
            if (!in_array($file_or_folder, ['.', '..'])) {
                if (is_dir($path . '/' . $file_or_folder)) {
                    $size += $this->getConsumedBytesInPath($path . '/' . $file_or_folder);
                } else {
                    $size += filesize($path . '/' . $file_or_folder);
                }
            }
        }
        return $size;
    }

    protected function getHumanReadableSize($size)
    {
        if ($size > 1024 * 1024 * 1024 * 1024) {
            $number = round($size / (1024 * 1024 * 1024 * 1024), 2);
            $unit = "TB";
        } elseif ($size > 1024 * 1024 * 1024) {
            $number = round($size / (1024 * 1024 * 1024), 2);
            $unit = "GB";
        } elseif ($size > 1024 * 1024) {
            $number = round($size / (1024 * 1024), 2);
            $unit = "MB";
        } else {
            $number = round($size / (1024), 2);
            $unit = "KB";
        }
        return $number . " " . $unit;
    }
}
