<?php

class MonitorController extends PluginController
{

    public function peek_action($server_id)
    {
        if (!$GLOBALS['perm']->have_perm('root') || !CheckStudip::displayMonitor()) {
            throw new AccessDeniedException();
        }
        Navigation::activateItem('/monitor/dashboard');
        $this->server = new CheckRemoteServer($server_id);
        $this->data = $this->server->fetch();
        $data = new CheckMonitoringData();
    }

    public function archive_action($data_id)
    {
        if (!$GLOBALS['perm']->have_perm('root') || !CheckStudip::displayMonitor()) {
            throw new AccessDeniedException();
        }
        $this->data = CheckMonitoringData::find($data_id);
        PageLayout::setTitle($this->data->server['name'].': '.sprintf(_('Report von %s'), date('H:i d.m.Y', $this->data['mkdate'])));
    }

    public function dashboard_action()
    {
        if (!$GLOBALS['perm']->have_perm('root') || !CheckStudip::displayMonitor()) {
            throw new AccessDeniedException();
        }
        Navigation::activateItem('/monitor/dashboard');
        PageLayout::addScript($this->plugin->getPluginURL().'/assets/monitor.js');
        PageLayout::addScript($this->plugin->getPluginURL().'/assets/apexcharts.min.js');
        PageLayout::addStylesheet($this->plugin->getPluginURL().'/assets/apexcharts.css');
        $this->max_slots = 20;
        $this->index = UserConfig::get($GLOBALS['user']->id)->CHECK_SELECTED_METRIC ?? 'users_online'; //load_percentage, mycourses_time


        if (Request::get('mode') === 'kiosk') {
            PageLayout::addStylesheet($this->plugin->getPluginURL().'/assets/kiosk.css');
        }

        $space = disk_free_space($GLOBALS['TMP_PATH']);
        if ($space > 1024 * 1024 * 1024 * 1024) {
            $this->fds_number = round($space / (1024 * 1024 * 1024 * 1024), 2);
            $this->fds_unit = "TB";
        } elseif ($space > 1024 * 1024) {
            $this->fds_number = round($space / (1024 * 1024 * 1024), 2);
            $this->fds_unit = "GB";
        } elseif ($space > 1024 * 1024) {
            $this->fds_number = round($space / (1024 * 1024), 2);
            $this->fds_unit = "MB";
        } else {
            $this->fds_number = round($space / (1024), 2);
            $this->fds_unit = "KB";
        }


        $minutes = DBManager::get()->query("
            SELECT `cronjobs_schedules`.`minute`
            FROM `cronjobs_schedules`
                LEFT JOIN `cronjobs_tasks` USING (`task_id`)
            WHERE `cronjobs_tasks`.`class` = 'CheckingServerJob'
            LIMIT 1
        ")->fetch(PDO::FETCH_COLUMN, 0);
        if ($minutes < 0) {
            $minutes = (-1) * $minutes;
        }
        $this->minutes = $minutes;

        $servers = CheckRemoteServer::findBySQL('1=1 ORDER BY name ASC');
        $data = [];

        $get_server_data = DBManager::get()->prepare("
            SELECT *
            FROM check_monitoring_data
            WHERE server_id = :server_id
                AND mkdate >= :min_time
                AND mkdate <= :max_time
            LIMIT 1
        ");
        foreach ($servers as $server) {
            $d = [];
            for ($i = 0; $i < $this->max_slots; $i++) {
                $get_server_data->execute([
                    'server_id' => $server->getId(),
                    'min_time' => time() - ($i * $minutes * 60) - ($minutes * 60 / 2),
                    'max_time' => time() - ($i * $minutes * 60) + ($minutes * 60 / 2)
                ]);
                $row = $get_server_data->fetch(PDO::FETCH_ASSOC);
                if ($row) {
                    $d[] = $row;
                }
            }
            $data[$server->getId()] = $d;
        }
        $this->data = $data;
        $warning = false;
        $texts = [
            'free_space_tmp' => _('Bei Server %s wird der TMP-Speicher knapp.')
        ];
        foreach (CheckStudip::getMetrics() as $metric) {
            $texts[$metric->getIndex()] = _('Server %s: ').$metric->getMessage();
        }
        $colors_default = [
            '#28497c', '#d60000', '#129c94', '#b02e7c', '#f26e00', '#6ead10', '#ffbd33', '#a85d45', '#008512'
        ];

        $this->series = CheckStudip::getGraphData();

        $server_ids = [];
        $this->colors = [];
        foreach ($servers as $server) {
            $server_ids[] = $server->id;
            if ($server['color']) {
                $this->colors[] = $server['color'];
            } else {
                $color = array_shift($colors_default);
                $this->colors[] = $color ?? '#000000';
            }
        }
        $points = CheckMonitoringPoint::findBySQL('`planned_time` >= UNIX_TIMESTAMP() - :slots ORDER BY `planned_time` ASC', [
            'slots' => $this->max_slots * 60 * 10
        ]);
        $this->categories = [];
        foreach ($points as $point) {
            $point_server_ids = [];
            foreach ($point->data as $data) {
                $point_server_ids[] = $data->server_id;
            }
            $this->categories[] = date("H:i", $point['planned_time'])." Uhr";
        }


        foreach ($this->data as $server_id => $set) {
            $server = CheckRemoteServer::find($server_id);
            if (!isset($set[0])) {
                continue;
            }
            $lastresponse = $set[0];
            if (isset($lastresponse) && $server['active']) {
                if (!$lastresponse['success']) {
                    PageLayout::postError(sprintf(_('Server %s ist nicht erreichbar!'), $server->name));
                    $warning = true;
                } else {
                    foreach ($server->alarms as $alarm) {
                        if (isset($lastresponse[$alarm['metric']])) {
                            if ($alarm['compare'] === '>=') {
                                if (floatval($lastresponse[$alarm['metric']]) >= floatval($alarm['value'])) {
                                    $type = $alarm['type'];
                                    PageLayout::postMessage(
                                        MessageBox::$type(sprintf($texts[$alarm['metric']], $server->name))
                                    );
                                }
                            }

                            if ($alarm['compare'] === '<=') {

                                if (floatval($lastresponse[$alarm['metric']]) <= floatval($alarm['value'])) {
                                    $type = $alarm['type'];
                                    PageLayout::postMessage(
                                        MessageBox::$type(sprintf($texts[$alarm['metric']], $server->name))
                                    );
                                }
                            }
                            if ($alarm['compare'] === '!=') {
                                if (trim($lastresponse[$alarm['metric']]) != trim($alarm['value'])) {
                                    $type = $alarm['type'];
                                    PageLayout::postMessage(
                                        MessageBox::$type(sprintf($texts[$alarm['metric']], $server->name))
                                    );
                                }
                            }
                            if ($alarm['compare'] === '=') {
                                if (trim($lastresponse[$alarm['metric']]) == trim($alarm['value'])) {
                                    $type = $alarm['type'];
                                    PageLayout::postMessage(
                                        MessageBox::$type(sprintf($texts[$alarm['metric']], $server->name))
                                    );
                                }
                            }
                        }
                    }
                }
            }
        }
        if ($warning) {
            PageLayout::setTitle('⚠ ALARM!');
        }
        $this->servers = $servers;
    }

    public function get_dashboard_action()
    {
        if (!$GLOBALS['perm']->have_perm('root') || !CheckStudip::displayMonitor()) {
            throw new AccessDeniedException();
        }
        $this->dashboard_action();
        $layout = $this->get_template_factory()->open('monitor/layout');
        $this->render_template('monitor/_dashboard', $layout);
    }

    public function change_metric_action()
    {
        if (Request::isPost() && Request::get('metric')) {
            UserConfig::get($GLOBALS['user']->id)->store("CHECK_SELECTED_METRIC", Request::get('metric'));
        }
        $this->redirect('monitor/dashboard');
    }
}
