<?php

class ServerController extends PluginController
{
    public function admin_action()
    {
        if (!$GLOBALS['perm']->have_perm('root') || !CheckStudip::displayMonitor()) {
            throw new AccessDeniedException();
        }
        Navigation::activateItem('/monitor/server');
        $this->servers = CheckRemoteServer::findBySQL('1=1 ORDER BY name ASC');
        $this->latest_check = [];
        foreach ($this->servers as $server) {
            $this->latest_check[$server->id] = CheckMonitoringData::findOneBySQL('`server_id` = ? ORDER BY `mkdate` DESC', [$server->id]);
        }
    }

    public function edit_action($server_id = null)
    {
        if (!$GLOBALS['perm']->have_perm('root') || !CheckStudip::displayMonitor()) {
            throw new AccessDeniedException();
        }
        PageLayout::setTitle($server_id === null ? _('Neuen Server hinzufügen') : _('Serverdaten bearbeiten'));
        Navigation::activateItem('/monitor/server');
        $this->server = new CheckRemoteServer($server_id);
        if (Request::isPost()) {
            if (Request::submitted("delete")) {
                $this->server->delete();
                PageLayout::postSuccess(_('Server wurden gelöscht'));
            } else {
                if ($this->server['url'] !== Request::get("url")) {
                    $contents = file_get_contents(Request::get("url"));
                    if ($contents) {
                        $contents = json_decode($contents, true);
                        if ($contents['installation_name']) {
                            $this->server['url'] = Request::get("url");
                            $this->server['name'] = $contents['installation_name'];
                        } else {
                            PageLayout::postError(_('URL enthält falsches Datenformat.'));
                        }
                    } else {
                        PageLayout::postError(_('URL ist nicht abrufbar.'));
                    }
                }
                $this->server['active'] = Request::get("active", 0);
                $this->server['ssl'] = Request::int("ssl", 0);
                $this->server['color'] = Request::get("color");
                $was_new = $this->server->isNew();
                $this->server->store();
                if ($was_new) {
                    foreach (CheckStudip::getDefaultAlarms() as $alarmdata) {
                        $alarm = new CheckRemoteServerAlarm();
                        $alarm->setData($alarmdata);
                        $alarm['server_id'] = $this->server->id;
                        $alarm->store();
                    }
                }
                PageLayout::postSuccess(_('Daten wurden gespeichert'));
            }
            $this->redirect("server/admin");
        }
    }

    public function alarms_action(CheckRemoteServer $server)
    {
        if (!$GLOBALS['perm']->have_perm('root') || !CheckStudip::displayMonitor()) {
            throw new AccessDeniedException();
        }
        $this->server = $server;
        PageLayout::setTitle(sprintf(_('Alarme für Server "%s" verwalten.'), $server['name']));
        $this->metrics = [
            'request_time',
            'internal_encoding',
            'internet_access'
        ];
        foreach (CheckStudip::getMetrics() as $metric) {
            $this->metrics[] = $metric->getIndex();
        }
        if (Request::isPost()) {
            if (Request::submitted('delete')) {
                CheckRemoteServerAlarm::find(Request::get('delete'))->delete();
            } else {
                foreach (Request::getArray('alarms') as $id => $data) {
                    if ($id !== 'new') {
                        $alarm = CheckRemoteServerAlarm::find($id);
                        $alarm->setData($data);
                        $alarm->store();
                    } elseif($data['metric']) {
                        $alarm = new CheckRemoteServerAlarm();
                        $alarm->setData($data);
                        $alarm['server_id'] = $server->id;
                        $alarm->store();
                    }
                }
            }
            PageLayout::postSuccess(_('Daten wurden gespeichert.'));
            $this->redirect('server/alarms/'.$server->id);
        }
    }

    public function filter_action()
    {
        if (!$GLOBALS['perm']->have_perm('root') && !CheckStudip::displayMonitor()) {
            throw new AccessDeniedException();
        }
        Navigation::activateItem('/monitor/server');
        $this->servers = CheckRemoteServer::findBySQL('1=1 ORDER BY name ASC');
        $this->latest_check = [];
        $this->versions = [];
        $this->php_versions = [];
        $this->plugins = [];
        foreach ($this->servers as $server) {
            $this->latest_check[$server->id] = CheckMonitoringData::findOneBySQL('`server_id` = ? ORDER BY `mkdate` DESC LIMIT 1', [$server->id]);
            $this->versions[] = $this->latest_check[$server->id]['studip_version'];
            $this->php_versions[] = $this->latest_check[$server->id]['php_version'];
            foreach (preg_split('/,\s*/', $this->latest_check[$server->id]['plugins']) as $plugin) {
                if (str_contains($plugin, ' ')) {
                    $plugin = explode(' ', $plugin);
                    $this->plugins[] = $plugin[0];
                } else {
                    $this->plugins[] = $plugin;
                }
            }
        }
        $this->versions = array_unique($this->versions);
        usort($this->versions, 'version_compare');
        $this->php_versions = array_unique($this->php_versions);
        usort($this->php_versions, 'version_compare');
        $this->plugins = array_unique($this->plugins);
        sort($this->plugins);
    }
}
