<?php

class TestsoapController extends PluginController
{

    public function overview_action()
    {
        Navigation::activateItem('/monitor/soap');
        $this->requests = CheckSoapRequest::findBySQL('1 ORDER BY name ASC');
    }

    public function compose_action()
    {
        Navigation::activateItem("/admin/locations/checkstudip");
        if (!$GLOBALS['perm']->have_perm("root")) {
            throw new AccessDeniedException();
        }
    }

    public function soapit_action()
    {
        Navigation::activateItem("/admin/locations/checkstudip");
        if (!$GLOBALS['perm']->have_perm("root") || !Request::isPost()) {
            throw new AccessDeniedException();
        }

        $client = new SoapClient(trim(Request::get("wsdl")), array(
            'connection_timeout' => 60 * 60,
            'trace' => 1,
            'exceptions' => 0,
            'cache_wsdl' => WSDL_CACHE_NONE,
            'features' => SOAP_SINGLE_ELEMENT_ARRAYS
        ));
        if (Request::get("username")) {
            $file = strtolower(substr(Request::get("wsdl"), strrpos(Request::get("wsdl"), "/") + 1));
            $soapHeaders = new SoapHeader($file, 'Header', array(
                'Login' => Request::get("username"),
                'Password' => htmlspecialchars(Request::get("password"))
            ));
            $client->__setSoapHeaders($soapHeaders);
        }
        if (is_soap_fault($client)) {
            PageLayout::postError("SOAP-Error: " . $client);
            $this->redirect(PluginEngine::getURL($this->plugin, $_POST, "testsoap/compose"));
            return;
        }

        $arguments = Request::get("json")
            ? json_decode(Request::get("json"), true)
            : null;

        if (Request::get("json") && !$arguments) {
            PageLayout::postError("Parameter sind nicht im JSON-Format.");
            $this->redirect(PluginEngine::getURL($this->plugin, $_POST, "testsoap/compose"));
            return;
        }

        $output = $client->__soapCall(
            Request::get("method"),
            $arguments
            //$options = null,
            //$input_headers = null,
            //$output_headers = null
        );

        ob_start();
        var_dump($output);
        $str = ob_get_clean();

        $params = array_merge($_POST, array('output' => $str));

        //var_dump($output);
        //$this->render_nothing();
        $this->redirect(PluginEngine::getURL($this->plugin, $params, "testsoap/compose"));

    }

    public function edit_action($request_id = null)
    {
        Navigation::activateItem("/admin/locations/checkstudip");
        if (!$GLOBALS['perm']->have_perm("root")) {
            throw new AccessDeniedException();
        }
        $this->request = new CheckSoapRequest($request_id);
    }

    public function save_action($request_id = null)
    {
        Navigation::activateItem("/admin/locations/checkstudip");
        if (!$GLOBALS['perm']->have_perm("root") || !Request::isPost()) {
            throw new AccessDeniedException();
        }
        $request = new CheckSoapRequest($request_id);
        $request['name'] = Request::get('name');
        $request['wsdl'] = trim(Request::get('wsdl'));
        $request['method'] = trim(Request::get('method'));
        $request['username'] = Request::get('username');
        $request['password'] = Request::get('password');
        $request['security_header'] = Request::get('security_header');
        $request['params'] = Request::get('json');
        $request['necessary_request'] = Request::int('necessary_request', 0);
        $request->store();
        PageLayout::postSuccess(_('Daten wurden gespeichert.'));
        $this->redirect('testsoap/overview');
    }

    public function save_as_request_action()
    {
        Navigation::activateItem("/admin/locations/checkstudip");
        if (!$GLOBALS['perm']->have_perm("root") || !Request::isPost()) {
            throw new AccessDeniedException();
        }
    }

    public function execute_action($request_id)
    {
        Navigation::activateItem("/admin/locations/checkstudip");
        if (!$GLOBALS['perm']->have_perm("root")) {
            throw new AccessDeniedException();
        }
        $this->request = CheckSoapRequest::find($request_id);
        PageLayout::setTitle(sprintf(_('Ausgabe von %s'), $this->request->name));
        $this->output = $this->request->execute();
        if ($this->output['error']) {
            PageLayout::postError(_('SOAP-Fehler'));
        }
    }

    public function delete_action($request_id)
    {
        Navigation::activateItem("/admin/locations/checkstudip");
        if (!$GLOBALS['perm']->have_perm("root") || !Request::isPost()) {
            throw new AccessDeniedException();
        }
        if ($request_id) {
            $request = CheckSoapRequest::find($request_id);
            $name = $request->name;
            $request->delete();
            PageLayout::postSuccess(sprintf(_('SOAP-Abfrage %s wurde gelöscht.'), $name));
        }
        $this->redirect('testsoap/overview');
    }
}
