<?php

class CheckSoapRequest extends SimpleORMap
{
    static protected function configure($config = array())
    {
        $config['db_table'] = 'check_soap_requests';
        parent::configure($config);
    }

    public function execute()
    {
        try {
            $client = new SoapClient(trim($this->wsdl), array(
                'connection_timeout' => 60 * 60,
                'trace' => 1,
                'exceptions' => 1,
                'cache_wsdl' => WSDL_CACHE_NONE,
                'features' => SOAP_SINGLE_ELEMENT_ARRAYS
            ));
        } catch (Exception $e) {
            return [
                'error' => 1,
                'message' => "SOAP-Error: " . $e->getMessage()
            ];
        }
        if ($this->username) {
            if ($this->security_header === 'wsse') {
                $headerbody = new \SoapVar('<wsse:Security SOAP-ENV:mustUnderstand="1" xmlns:wsse="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"><wsse:UsernameToken><wsse:Username>' . \htmlspecialchars($this->username) . '</wsse:Username><wsse:Password Type="http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordText">' . \htmlspecialchars($this->password) . '</wsse:Password></wsse:UsernameToken></wsse:Security>', \XSD_ANYXML);
                $soapHeaders = new \SoapHeader(
                    "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd",
                    'Security',
                    $headerbody
                );
            } else {
                $file = strtolower(substr($this->wsdl, strrpos($this->wsdl, "/") + 1));
                $soapHeaders = new SoapHeader($file, 'Header', array(
                    'Login' => $this->username,
                    'Password' => $this->password
                ));
            }
            $client->__setSoapHeaders($soapHeaders);
        }
        if (is_soap_fault($client)) {
            return [
                'error' => 1,
                'message' => "SOAP-Error: " . $client
            ];
        }

        $arguments = $this->params
            ? json_decode($this->params, true)
            : null;

        if ($this->params && !$arguments) {
            return [
                'error' => 1,
                'message' => "Parameter sind nicht im JSON-Format."
            ];
        }

        try {
            $output = $client->__soapCall(
                $this->method,
                $arguments
            );
        } catch (Exception $e) {
            return [
                'error' => 1,
                'message' => "SOAP-Error: ".$e->getMessage()
            ];
        }

        return [
            'error' => 0,
            'message' => print_r($output, true)
        ];
    }
}
