<?php

namespace CheckStudip;

class Cronjobs implements \CheckStudipMetric
{
    public function getName()
    {
        return _('Cronjobs aktiv');
    }

    public function getDescription()
    {
        return null;
    }

    public function getIndex()
    {
        return 'cronjobs_working';
    }

    public function measure()
    {
        if (!\Config::get()->CRONJOBS_ENABLE) {
            return null;
        } else {
            if (\StudipVersion::newerThan('5.99')) {
                $cronjobs = \CronjobSchedule::findBySQL("`active` = 1");
            } else {
                $cronjobs = \CronjobSchedule::findBySQL("`active` = 1 AND `type` = 'periodic' ");
            }
            $lastExecution = null;
            $lastcronjob = null;
            $escalation_time = \Config::get()->CRONJOBS_ESCALATION ?: 60 * 15;
            foreach ($cronjobs as $cronjob) {
                $execution = $this->cronjobLastExecution($cronjob);
                if ($execution !== null && $execution > $lastExecution) {
                    $lastExecution = $execution;
                    $lastcronjob = $cronjob;
                }
            }
            return $lastcronjob['last_execution'] >= $lastExecution - $escalation_time ? 1 : 0;
        }
    }



    public function defaultAlarm()
    {
        return [
            'type' => 'warning',
            'compare' => '=',
            'value' => '0'
        ];
    }

    public function getMessage()
    {
        return _('Cronjobs scheinen nicht zu laufen.');
    }


    protected function cronjobLastExecution(\CronjobSchedule $cronjob)
    {
        $result  = time() - 60 * 5;
        $result -= $result % 60;

        $i = 24 * 60; // Maximum: A day
        $offset = 60;

        do {
            $result -= $offset;

            // TODO: Performance - Adjust result according to conditions
            // See http://coderzone.org/library/PHP-PHP-Cron-Parser-Class_1084.htm
            $valid  = $this->cronjobTestTimestamp($result, $cronjob->minute, 'i')
                && $this->cronjobTestTimestamp($result, $cronjob->hour, 'H')
                && $this->cronjobTestTimestamp($result, $cronjob->day, 'd')
                && $this->cronjobTestTimestamp($result, $cronjob->month, 'm')
                && $this->cronjobTestTimestamp($result, $cronjob->day_of_week, 'N');

        } while (!$valid && $i-- > 0);

        return $i > 0 ? $result : null;
    }

    protected function cronjobTestTimestamp($timestamp, $condition, $format)
    {
        if ($condition === null) {
            return true;
        }

        $probe     = (int) date($format, $timestamp);
        $condition = (int) $condition;

        if ($condition < 0) {
            return ($probe % abs($condition)) === 0;
        }

        return $probe === $condition;
    }
}
