<?php

namespace CheckStudip;
class MyCoursesTime implements \CheckStudipMetric
{
    public function getIndex()
    {
        return 'mycourses_time';
    }

    public function getName()
    {
        return _('Meine-Veranstaltungen Ladedauer');
    }

    public function getDescription()
    {
        return _('Es wird die Zeit der Meine-Veranstaltungsseite simuliert. Dazu wird eine zufällige Person ausgewählt, die im letzten Monat aktiv war und überdurchschnittlich viele Veranstaltungen hat (allerdings nicht die Nutzer mit den allermeisten Veranstaltungen). Für diese Person werden einmal alle Icons der Meine-VA berechnet und davon die Zeit gestoppt.');
    }

    public function defaultAlarm()
    {
        return [
            'type' => 'warning',
            'compare' => '>=',
            'value' => 2
        ];
    }

    public function getMessage()
    {
        return _('Die Meine-Veranstaltungen-Seite dauert lange.');
    }


    public function measure()
    {
        //first we search for an active user (last 30 days online) that has a typical amount of courses
        $number_of_users = \DBManager::get()->prepare("
            SELECT COUNT(*) AS counted FROM (
                SELECT COUNT(*) AS courses
                FROM user_online
                    INNER JOIN seminar_user ON (seminar_user.user_id = user_online.user_id)
                WHERE user_online.last_lifesign >= UNIX_TIMESTAMP() - 86400 * 30
                GROUP BY user_online.user_id
            ) AS all_user_all_courses
        ");
        $number_of_users->execute();
        $number = $number_of_users->fetch(\PDO::FETCH_COLUMN, 0);
        $number = floor($number / 10);

        $get_user_id = \DBManager::get()->prepare("
            SELECT user_online.user_id, COUNT(*) AS courses
            FROM user_online
                INNER JOIN seminar_user ON (seminar_user.user_id = user_online.user_id)
            WHERE user_online.last_lifesign >= UNIX_TIMESTAMP() - 86400 * 30
            GROUP BY user_online.user_id
            ORDER BY courses DESC
            LIMIT ".$number.", 1
        ");
        $get_user_id->execute();
        $user_id = $get_user_id->fetch(\PDO::FETCH_COLUMN, 0); // a typical active user

        //Now we do our test with this typical but random user
        $start = \microtime(true);
        $courses = \Course::findThru($user_id, [
            'thru_table'        => 'seminar_user',
            'thru_key'          => 'user_id',
            'thru_assoc_key'    => 'seminar_id',
            'assoc_foreign_key' => 'seminar_id'
        ]);
        foreach ($courses as $course) {
            foreach ($course->tools as $tool) {
                if ($module = $tool->getStudipModule()) {
                    $last_visit = \object_get_visit($course->id, $module->getPluginId());
                    $module->getIconNavigation($course->id, $last_visit, $user_id);
                }
            }
        }

        $end = \microtime(true);
        return ($end - $start);
    }
}
