<?php

namespace CheckStudip;

class UnwritableFolders implements \CheckStudipMetric
{
    public function getName()
    {
        return _('Nichtschreibbare Ordner');
    }

    public function getIndex()
    {
        return 'nonwritable_folders';
    }

    public function getDescription()
    {
        return _('Ordner, die beschreibbar sein sollen, sind data/user_doc, data/assets_cache, data/oer, data/oer_logos, public/plugins_packages, public/pictures für Veranstaltungen, Nutzer, Einrichtungen und Hintergrundgrafiken.');
    }

    public function measure()
    {
        $writable_folders = array(
            $GLOBALS['UPLOAD_PATH'],  // "data/user_doc"
            $GLOBALS['PLUGIN_ASSETS_PATH'], //"data/assets_cache"
            $GLOBALS['OER_PATH'], //"data/oer"
            $GLOBALS['OER_LOGOS_PATH'], //"data/oer_logos"
            $GLOBALS['PLUGINS_PATH'], //"public/plugins_packages"
            $GLOBALS['DYNAMIC_CONTENT_PATH'], // "public/pictures"
            $GLOBALS['DYNAMIC_CONTENT_PATH']."/course",
            $GLOBALS['DYNAMIC_CONTENT_PATH']."/user",
            $GLOBALS['DYNAMIC_CONTENT_PATH']."/institute",
            $GLOBALS['DYNAMIC_CONTENT_PATH']."/loginbackgrounds"
        );
        $output['nonwritable_folders'] = [];
        foreach ($writable_folders as $folder) {
            $folderpath = $folder[0] === "/" ? $folder : $GLOBALS['STUDIP_BASE_PATH']."/".$folder;
            if (!\is_writable($folderpath)) {
                $output['nonwritable_folders'][] = $folder;
            }
        }
        return \implode(', ', $output['nonwritable_folders']);
    }

    public function defaultAlarm()
    {
        return [
            'type' => 'error',
            'compare' => '=',
            'value' => 'no'
        ];
    }

    public function getMessage()
    {
        return _('Einer oder mehrere Ordner sind nicht schreibbar für den Webserver.');
    }
}
