<?php
class AddMonitoring extends Migration
{
	function up()
    {
        Config::get()->create("CHECKSTUDIP_MONITOR_ROLE", array(
            'value' => 'root',
            'type' => "string",
            'range' => "global",
            'section' => "CheckStudip",
            'description' => "What roles should see the monitor? autor, admin, root, pluginrole?"
        ));
        DBManager::get()->exec("
            CREATE TABLE `check_remote_servers` (
                `server_id` char(32) NOT NULL,
                `name` varchar(128) DEFAULT NULL,
                `url` varchar(256) NOT NULL,
                `chdate` int(11) DEFAULT NULL,
                `mkdate` int(11) DEFAULT NULL,
                PRIMARY KEY (`server_id`)
            ) ENGINE=InnoDB;
        ");
        DBManager::get()->exec("
            CREATE TABLE `check_monitoring_data` (
                `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
                `server_id` char(32)  DEFAULT NULL,
                `json` text DEFAULT NULL,
                `chdate` int(11) DEFAULT NULL,
                `mkdate` int(11) DEFAULT NULL,
                PRIMARY KEY (`id`)
            ) ENGINE=InnoDB;
        ");

        $new_job = array(
            'filename'    => 'public/plugins_packages/data-quest/CheckStudip/checking.cronjob.php',
            'class'       => 'CheckingServerJob',
            'priority'    => 'normal',
            'minute'      => '-10'
        );

        if (StudipVersion::newerThan('5.99')) {
            $query = "INSERT IGNORE INTO `cronjobs_tasks`
                    (`task_id`, `filename`, `class`, `active`)
                  VALUES (:task_id, :filename, :class, 1)";
            $task_statement = DBManager::get()->prepare($query);

            $query = "INSERT IGNORE INTO `cronjobs_schedules`
                    (`schedule_id`, `task_id`, `parameters`,
                     `minute`, `mkdate`, `chdate`,
                     `last_result`, `active`)
                  VALUES (:schedule_id, :task_id, '[]',
                          :minute, UNIX_TIMESTAMP(), UNIX_TIMESTAMP(),
                          NULL, 1)";
            $schedule_statement = DBManager::get()->prepare($query);


            $task_id = md5(uniqid('task', true));

            $task_statement->execute(array(
                ':task_id' => $task_id,
                ':filename' => $new_job['filename'],
                ':class' => $new_job['class'],
            ));

            $schedule_id = md5(uniqid('schedule', true));
            $schedule_statement->execute(array(
                ':schedule_id' => $schedule_id,
                ':task_id' => $task_id,
                ':minute' => $new_job['minute'],
            ));
        } else {
            $query = "INSERT IGNORE INTO `cronjobs_tasks`
                    (`task_id`, `filename`, `class`, `active`)
                  VALUES (:task_id, :filename, :class, 1)";
            $task_statement = DBManager::get()->prepare($query);

            $query = "INSERT IGNORE INTO `cronjobs_schedules`
                    (`schedule_id`, `task_id`, `parameters`, `priority`,
                     `type`, `minute`, `mkdate`, `chdate`,
                     `last_result`, `active`)
                  VALUES (:schedule_id, :task_id, '[]', :priority, 'periodic',
                          :minute, UNIX_TIMESTAMP(), UNIX_TIMESTAMP(),
                          NULL, 1)";
            $schedule_statement = DBManager::get()->prepare($query);


            $task_id = md5(uniqid('task', true));

            $task_statement->execute(array(
                ':task_id' => $task_id,
                ':filename' => $new_job['filename'],
                ':class' => $new_job['class'],
            ));

            $schedule_id = md5(uniqid('schedule', true));
            $schedule_statement->execute(array(
                ':schedule_id' => $schedule_id,
                ':task_id' => $task_id,
                ':priority' => $new_job['priority'],
                ':minute' => $new_job['minute'],
            ));
        }
	}

    function down()
    {
        Config::get()->delete("CHECKSTUDIP_MONITOR_ROLE");
        DBManager::get()->exec("
            DROP TABLE `check_remote_servers`
        ");
        DBManager::get()->exec("
            DROP TABLE `check_monitoring_data`
        ");

        $statement = DBManager::get()->prepare("SELECT task_id FROM cronjobs_tasks WHERE `class` = 'CheckingServerJob'");
        $statement->execute();
        $task_id = $statement->fetch(PDO::FETCH_COLUMN, 0);
        DBManager::get()->prepare("DELETE FROM cronjobs_tasks WHERE `class` = 'CheckingServerJob'");
        $statement = DBManager::get()->prepare("DELETE FROM `cronjobs_schedules` WHERE task_id = ?");
        $statement->execute([$task_id]);
    }
}
