<?php
class AddAlarmConfig extends Migration
{
	function up()
    {

        DBManager::get()->exec("
            CREATE TABLE `check_remote_servers_alarms` (
                `id` int(11) unsigned NOT NULL AUTO_INCREMENT,
                `server_id` char(32) NOT NULL,
                `type` enum('error','warning','info') NOT NULL,
                `metric` varchar(50) NOT NULL,
                `compare` varchar(2) NOT NULL,
                `value` varchar(100) NOT NULL,
                `chdate` int(11) DEFAULT NULL,
                `mkdate` int(11) DEFAULT NULL,
                PRIMARY KEY (`id`),
                KEY `server_id` (`server_id`)
            )
        ");

        $alarms = [
            [
                'metric' => 'internal_encoding',
                'compare' => '!=',
                'value' => 'UTF-8',
                'type' => 'error'
            ],
            [
                'metric' => 'free_space_tmp',
                'compare' => '<=',
                'value' => Config::get()->DISK_FREE_SPACE_ALERT,
                'type' => 'warning'
            ],
            [
                'metric' => 'free_space_data',
                'compare' => '<=',
                'value' => 1024 * 1024 * 1024 * 5,
                'type' => 'warning'
            ],
            [
                'metric' => 'mycourses_time',
                'compare' => '>=',
                'value' => 0.9,
                'type' => 'warning'
            ],
            [
                'metric' => 'load_percentage',
                'compare' => '>=',
                'value' => 80,
                'type' => 'warning'
            ],
            [
                'metric' => 'internet_access',
                'compare' => '=',
                'value' => 0,
                'type' => 'warning'
            ],
            [
                'metric' => 'nonwritable_folders',
                'compare' => '!=',
                'value' => '',
                'type' => 'error'
            ],
            [
                'metric' => 'failed_soap_requests',
                'compare' => '!=',
                'value' => '',
                'type' => 'error'
            ]
        ];

        $server_statement = DBManager::get()->prepare("
            SELECT * FROM `check_remote_servers`
        ");
        $server_statement->execute();
        $alarm_insert = DBManager::get()->prepare("
            INSERT INTO `check_remote_servers_alarms`
            SET `metric` = :metric,
                `server_id` = :server_id,
                `type` = :type,
                `value` = :value,
                `compare` = :compare,
                `chdate` = UNIX_TIMESTAMP(),
                `mkdate` = UNIX_TIMESTAMP()
        ");
        while ($server = $server_statement->fetch(PDO::FETCH_ASSOC)) {
            foreach ($alarms as $alarm) {
                $alarm['server_id'] = $server['server_id'];
                $alarm_insert->execute($alarm);
            }
        }
        Config::get()->delete("DISK_FREE_SPACE_ALERT");
	}

    function down()
    {
        Config::get()->create("DISK_FREE_SPACE_ALERT", array(
            'value' => 1024 * 1024 * 1024 * 2,
            'type' => "integer",
            'range' => "global",
            'section' => "CheckStudip",
            'description' => "Root should be warned if the free disk space in /TMP is lower than x bytes?"
        ));
        DBManager::get()->exec("
            DROP TABLE `check_remote_servers_alarms`
        ");
    }
}
